////////////////////////////////////////////////////////////////////////////////
//==============================================================================
// Principal.cpp   - Compile with Borland C++ builder -
//==============================================================================
// modified     | ver.  |  by                  |         description
//--------------|-------|----------------------|--------------------------------
// june 08 2003 | 0.91  | Matthieu WEBER       | 2nd official beta release
//              |       |mweber_29200@yahoo.com| 
//==============================================================================
// Comments:
//
//  English translation of variable's names in progress...
//
////////////////////////////////////////////////////////////////////////////////

//---------------------------------------------------------------------------
#include <vcl.h>
#include <string.h>
#include <stdio.h>
#include <Registry.hpp>
#include <mmsystem.h>
#include <tapi.h>

#pragma hdrstop
#include "Principal.h"
#include "Config.h"
#include "math.h"

#pragma package(smart_init)
#pragma link "DefautJaugeMat"
#pragma resource "*.dfm"

/***************************************************************************/
/* Variables definition                                                    */
/***************************************************************************/

TPrinc *Princ;


/***************************************************************************/
/* Functions                                                               */
/***************************************************************************/

// Users functions

//--------------------------------------------------------------------------
// Converts drive capacity into ANSISTRING
AnsiString __fastcall TPrinc::ConvertTaille(__int64 size) // Size in bytes
  {
  if (size >= 1099511627776)
    {
    return(FormatFloat("###0.# To", size / (float)1099511627776));
    }
  if (size >= 1073741824)
    {
    return(FormatFloat("###0.# Go", size / (float)1073741824));
    }
  if (size >= 1048576)
    {
    return(FormatFloat("###0.# Mo", size / (float)1048576));
    }
  if (size >= 1024)
    {
    return(FormatFloat("###0.# ko", size / (float)1024));
    }
  return(IntToStr(size) + " o");
  }

//--------------------------------------------------------------------------
// Converts seconds into hours minutes format
AnsiString __fastcall TPrinc::ConvertHeure(__int64 seconds)
  {
  short int minutes;
  short int hours;

  minutes = 60 * ((seconds / (float)3600)  - (int)(seconds / 3600));
  hours   = 24 * ((seconds / (float)86400) - (int)(seconds / 86400));
  return(FormatFloat("## H ;; ", hours) + FormatFloat("#0'\"'", minutes));
  }
//---------------------------------------------------------------------------

// TJaugeMat component 

//---------------------------------------------------------------------------
// Composant TJaugeMat : Constructor extends from TDefautJaugeMat
__fastcall TJaugeMat::TJaugeMat(TComponent* Owner)
  : TDefautJaugeMat(Owner)
  {
  Lecteur = 'C';
  lstrcpy(Chemin, "C:\\");
  }

//---------------------------------------------------------------------------

// Main window

//---------------------------------------------------------------------------
// Main window constructor
__fastcall TPrinc::TPrinc(TComponent* Owner)
        : TForm(Owner)
{
}

//---------------------------------------------------------------------------
// OnDestroy event called at program's ending
void __fastcall TPrinc::FormDestroy(TObject *Sender)
  {
  // free memory
  CleanUp();
  delete GaugeList;
  delete SoundCtrlList;

  // Save configuration in the system registery
  Save_cfg();
  }

//---------------------------------------------------------------------------
// Timer1 event occurence: when occurs: asks to refresh gauges
 void __fastcall TPrinc::Timer1Timer(TObject *Sender)
  {
  int i;
  if (GaugeList->Count > 0)
    {
    for(i = 0; i < GaugeList->Count; i++)
      {
      Gauge = (TJaugeMat*) GaugeList->Items[i];
      if (Gauge->Lecteur == '@')
        {
        UpdateMemory(Gauge);    // Refresh RAM
        continue;
        }
      if (Gauge->Lecteur == '#')
        {
        UpdateBattery(Gauge);   // Refresh battery status
        continue;
        }

// We don't refresh floppy disks: No infos are displayed (says dbl. click to
// refresh), until the user doubles ckick

// "tag" property is used to inform this function the kind of drive passed in
// parameter (this tag is in fact drive's icon number)
      if ((Gauge->Lecteur != 'A') && (Gauge->Lecteur != 'B'))
        {
        // We refresh all but floppy disks
        UpdateDrive(Gauge);
        }
      }
    }
  }

//---------------------------------------------------------------------------
// Timer2 event occurence=> when occurs: asks to hide the main window
void __fastcall TPrinc::Timer2Timer(TObject *Sender)
  {
  int i;

  switch (ZoneMasque)
    {
    case HIDE_RIGHT:
      Left = Screen->Width + ClientWidth - Width + 1;
    break;
    case HIDE_LEFT:
      Left = -ClientWidth - 1;
    break;
    case HIDE_TOP:
      Top = -ClientHeight - 1;
    break;
    case HIDE_BOTTOM:
      Top = Screen->Height + ClientHeight - Height + 1;
    break;
    }
  Timer2->Enabled = false;
  OnMouseMove = FormMouseMove;
  if (GaugeList->Count > 0)
    {
    for(i = 0; i < GaugeList->Count; i++)
      {
      ((TJaugeMat*) GaugeList->Items[i])->OnMouseMove = FormMouseMove;
      }
    }
  }

//---------------------------------------------------------------------------
// Displays window's explorer or file browser when double clicking on one of
// the drives
void __fastcall TPrinc::FormDblClick(TObject *Sender)

  {
  AnsiString selection;

  // Retrive the name of the drive linked with the calling gauge
  selection = ((TJaugeMat*)Sender)->Lecteur;

  if ((selection == "A")||(selection == "B"))  // If selection = floppy drive
    {
    UpdateDrive((TJaugeMat*)Sender);
    }

  if (Explorateur)
    {
    ShellExecute(Handle, "explore", strcat(selection.c_str(), ":"), NULL, NULL, SW_SHOWNORMAL);
    }
  else
    {
    ShellExecute(Handle, "open", strcat(selection.c_str(), ":"), NULL, NULL, SW_SHOWNORMAL);
    }
  }

//---------------------------------------------------------------------------
// Displays settings window if left click on a drive; right click refresh drives
void __fastcall TPrinc::OnMouseDown(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
int i;
// Right click
  if (Shift.Contains(ssRight))
    {
    // If the setting windows got OK button pressed
    if (Configuration->ShowModal() == mrOk)
      {
      // Save configuration in the system registery
      Save_cfg();
      // Delete old gauges and controls
      CleanUp();
      // Set up new display settings
      Affichage = Configuration->NouvAffichage;
      // Displays new jauges and controls
      DisplaysControls();
      Timer2->Enabled = bHideMainWindow;
      }
    }
    else
    // If left click
      {
      // Call Timer1 to refresh immediately datas (as user's request)
      Timer1Timer(Princ);
      }
}
//---------------------------------------------------------------------------
void __fastcall TPrinc::FormOnChange(TObject *Sender)
{
// The user has moved a sound potentiometer...  Most of theise paramerter
// are not implemented yet

// Adjust audio volume
    int selection;
    selection = ((TTrackBar*)Sender)->Tag;

    switch (selection)
      {
      case 1: // Main volume is set
        {
        unsigned long tmp = 0;
        tmp = ((TTrackBar*)Sender)->Position * 655;
        //mainSetVolume(NULL, (tmp << 16) + tmp);
        if(((Princ->Affichage & DISPLAY_STATUS) != 0))  // if (StatusBar != NULL)
          {
          StatusBar->SimpleText = (AnsiString)txt_volume + " " + \
          (AnsiString)((TTrackBar*)Sender)->Position + " " +(AnsiString) "%";
          }
        break;
        }
      case 2:  // Wav volume is set
        {
        unsigned long tmp = 0;
        tmp = ((TTrackBar*)Sender)->Position * 655;
        waveOutSetVolume(NULL, (tmp << 16) + tmp);
        if(((Princ->Affichage & DISPLAY_STATUS) != 0))  // if (StatusBar != NULL)
          {
          StatusBar->SimpleText = (AnsiString)txt_vol_wav + " " + \
          (AnsiString)((TTrackBar*)Sender)->Position + " " +(AnsiString) "%";
          }
        break;
        }
      case 3:  // MIDI volume is set
        {
        unsigned long tmp = 0;
        tmp = ((TTrackBar*)Sender)->Position * 655;
        midiOutSetVolume(NULL, (tmp << 16) + tmp);

        if(((Princ->Affichage & DISPLAY_STATUS) != 0))  // if (StatusBar != NULL)
          {
          StatusBar->SimpleText = (AnsiString)txt_vol_midi + " " + \
          (AnsiString)((TTrackBar*)Sender)->Position + " " +(AnsiString) "%";
          }
        break;
      }
      case 4:  // CD volume is set
        {
        unsigned long tmp = 0;
        tmp = ((TTrackBar*)Sender)->Position * 655;
        //cdSetVolume(NULL, (tmp << 16) + tmp);
        if(((Princ->Affichage & DISPLAY_STATUS) != 0))  // if (StatusBar != NULL)
          {
          StatusBar->SimpleText = (AnsiString)txt_vol_cd + " " + \
          (AnsiString)((TTrackBar*)Sender)->Position + " " +(AnsiString) "%";
          }
        break;
        }
      case 5: // Auxiliary input volume is set
        {
        unsigned long tmp = 0;
        tmp = ((TTrackBar*)Sender)->Position * 655;
        auxSetVolume(NULL, (tmp << 16) + tmp);

        if(((Princ->Affichage & DISPLAY_STATUS) != 0))  // if (StatusBar != NULL)
          {
          StatusBar->SimpleText = (AnsiString)txt_vol_aux + " " + \
          (AnsiString)((TTrackBar*)Sender)->Position + " " +(AnsiString) "%";
          }
        break;
        }
      case 6:  // Microphone volume is set
        {
        unsigned long tmp = 0;
        tmp = ((TTrackBar*)Sender)->Position * 655;
        //micSetVolume(NULL, (tmp << 16) + tmp);
        if(((Princ->Affichage & DISPLAY_STATUS) != 0))  // if (StatusBar != NULL)
          {
          StatusBar->SimpleText = (AnsiString)txt_vol_micro + " " + \
          (AnsiString)((TTrackBar*)Sender)->Position + " " +(AnsiString) "%";
          }
        break;
        }
      }
      // Stop and restart the 1 second Timer attached to the status bar in
      // order to see volume change exactly during one second after the last
      // user's volume change

      StatusBarTimer->Enabled = false;
      StatusBarTimer->Enabled = true;
}

//---------------------------------------------------------------------------
// OnMouseMove event of the main windows, to show the hidden window
void __fastcall TPrinc::FormMouseMove(TObject *Sender, TShiftState Shift,
      int X, int Y)
{
  UnHide(bHideMainWindow);
}

//---------------------------------------------------------------------------
// Add gauges or controls and let them be displayed according to the user
// settings. This function is called at the begining of the application and
// every times we need to rescan and display the gauges and controls.
void __fastcall TPrinc::DisplaysControls()
{
  // Set main window's size to the smallest value
  MainWindowHeight = 27;
  char Temp[4];
  DWORD Lecteurs;
  unsigned int NumLettre = 0;

  // Specifiy main window's horizontal size (+5 is the external bevel)
  Width = TAILLE_JAUGE_LARG + 5;


  // If showing RAM gauge
  if ((Affichage & DISPLAY_MEMORY) != 0)
    {
    AddGauge('@', 5);
    }

  // If showing battery charge gauge
  if ((Affichage & DISPLAY_BATTERY) != 0)
    {// 0 => Battery icon was inserted manually in the icon list "ImageList1"
    AddGauge('#', 0);
    }

  // Retrive system's drive handlers then display them
  Lecteurs = GetLogicalDrives();
  lstrcpy(Temp, "A:\\");
  while (Lecteurs > 0)
    {
    if (Lecteurs & 1 == 1)
      {
      // Retrive data mask
      Temp[0]= NumLettre + 'A';
      switch(GetDriveType(Temp))
        {
        case 0:
          MessageBox(Handle, Temp, txt_unknown_drive_type.c_str(), MB_OK);
          break;
        case 1:
          // Unable to find the drive path
          break;
        case DRIVE_REMOVABLE:
          if ((Affichage & DISPLAY_REMOVABLE_DRV) != 0)
            AddGauge(Temp[0], 1);
          break;
        case DRIVE_FIXED:
          if ((Affichage & DISPLAY_FIXED_DRV) != 0)
            AddGauge(Temp[0], 2);
          break;
        case DRIVE_REMOTE:
          if ((Affichage & DISPLAY_NET_DRV) != 0)
            AddGauge(Temp[0], 3);
          break;
        case DRIVE_CDROM:
          if ((Affichage & DISPLAY_CDROM) != 0)
            AddGauge(Temp[0], 4);
          break;
        case DRIVE_RAMDISK:
          if ((Affichage & DISPLAY_RAMDISK) != 0)
            AddGauge(Temp[0], 5);
          break;
        }
      }
      NumLettre++;
      Lecteurs = Lecteurs >> 1;
    }

// Display (optionnal) audio controls

// Main volume control
  if ((Affichage & DISPLAY_MAIN_VOL) != 0)
    {
    AddSoundControl("Vol.", 1);
    }
// Wav volume control
  if ((Affichage & DISPLAY_WAV_VOL) != 0)
    {
    AddSoundControl("Wav", 2);
    }
// MIDI volume control
  if ((Affichage & DISPLAY_MIDI_VOL) != 0)
    {
    AddSoundControl("MIDI", 3);
    }
// CD volume control
  if ((Affichage & DISPLAY_CD_VOL) != 0)
    {
    AddSoundControl("CD", 4);
    }
// Auxiliary input volume control
  if ((Affichage & DISPLAY_AUX_VOL) != 0)
    {
    AddSoundControl("Aux.", 5);
    }
// Microphone volume control
  if ((Affichage & DISPLAY_MIC_VOL) != 0)
    {
    AddSoundControl("Mic.", 6);
    }

// Display status bar (optionnal)

  if ((Affichage & DISPLAY_STATUS) !=0)
    {
    StatusBar = new TStatusBar("");
    StatusBar->Parent = Princ;
    StatusBar->Font->Name = "Small Fonts";
    StatusBar->Font->Height = -8;
    StatusBar->Height = 15;

// Adjust main window's vertical size to display status bar
    MainWindowHeight += StatusBar->Height;
    }
  MainWindowHeight -= 4;
  Height = MainWindowHeight;
// Call Timer1Timer to display immediately the (first collected) datas
  Timer1Timer(Princ);
}

//---------------------------------------------------------------------------
// Adds gauges after thoses already displayed
void __fastcall TPrinc::AddGauge(char Lecteur, unsigned char IndiceIco)
  {
  Graphics::TBitmap* temp;
  char NomVol[64];

  Gauge= new TJaugeMat(Princ);
  GaugeList->Add(Gauge);
  Gauge->Parent = Princ;
  Gauge->Font->Name = "Small Fonts";
  Gauge->Font->Height = -8;
  Gauge->Left = 0;
  Gauge->Top = (GaugeList->Count - 1) * (TAILLE_JAUGE_HAUT);
  Gauge->Height = TAILLE_JAUGE_HAUT;
  Gauge->Width = TAILLE_JAUGE_LARG;

  // Adjust main window's vertical size to add the new control
  MainWindowHeight += Gauge->Height;

  // Here, the "tag" property is used to send the kind of drive in parameters
  // (via his icon ID) to the function "UpdateDrive"
  Gauge->Tag = IndiceIco;

  if (Lecteur == '@')
    Gauge->Label = txt_memoire;               // If current gauge is RAM
  else
    if ((Lecteur != '@') && (Lecteur != '#')) // If current gauge is a drive
      {
      Gauge->Chemin[0] = Lecteur;             // displays the path
      Gauge->OnDblClick = FormDblClick;       // double click enabled
      }

    Gauge->Lecteur = Lecteur;                 // In all cases...
    Gauge->Pourcentage = true;
    Gauge->OnMouseDown = OnMouseDown;         // ...Right click enabled to acess config window

    if ((Lecteur != '@') && (Lecteur != '#'))
      {
      Gauge->Pourcentage = false;  // Hide "%" if there is no disk in drive
      // Displays a text asking user to click to display the floppy drive's infos
      Gauge->Label = AnsiString("( ") + AnsiString(Gauge->Lecteur) + AnsiString(" ) ") + txt_rafraich;
      }

    // If shading display (texture display)
    if (Texture)
      {
      Graphics::TBitmap *temp = new Graphics::TBitmap();
      // Shading choice
      ImageList2->GetBitmap(Type_Texture, temp);
      if (InvertDisp)
        {
        // We invert the bitmap to show an inverted jauge (not implemented yet)
        ;
        }
      Gauge->Bitmap = temp;
      delete temp;
      }

    Gauge->Texture = Texture;
    ImageList1->GetIcon(IndiceIco, Gauge->Icone);
    }
//---------------------------------------------------------------------------
// Adds others controls after thoses already displayed
void __fastcall TPrinc::AddSoundControl(char* Controle, unsigned char index)
  {
  Volume = new TTrackBar(Princ);
  SoundCtrlList->Add(Volume);
  Volume->Parent = Princ;
  Volume->Left = 0;
  Volume->Top = (GaugeList->Count) * TAILLE_JAUGE_HAUT + \
        (SoundCtrlList->Count - 1) * (TAILLE_VOLUME+2) + 2;
  Volume->Tag = index;

  // Volume control
   Volume->Left = 20;
   Volume->Height = 15;
   Volume->Width = 87;
   Volume->ThumbLength = 10;
   Volume->Max = 100;
   Volume->Frequency = 10;
   Volume->LineSize = 5;
   Volume->PageSize = 8;
   Volume->TickMarks = Comctrls::tmBoth;
   Volume->TickStyle = tsNone;
   Volume->OnChange = FormOnChange;  // Attaching on chnage => volume event

   VolCtrlBevel = new TBevel(Princ);
   VolCtrlBevel->Shape =  bsFrame;
   VolCtrlBevel->Parent = Princ;
   VolCtrlBevel->Left = 0;
   VolCtrlBevel->Top = (GaugeList->Count) * TAILLE_JAUGE_HAUT + \
        (SoundCtrlList->Count - 1) * (TAILLE_VOLUME+2);
   VolCtrlBevel->Height = 19;
   VolCtrlBevel->Width = TAILLE_JAUGE_LARG;

   VolTxt = new TLabel(Princ);
   VolTxt->Caption = "Vol.";
   VolTxt->Font->Name = "Small Fonts";
   VolTxt->Font->Height = -8;
   VolTxt->Parent = Princ;
   VolTxt->Left = 3;
   VolTxt->Top = 5 + (GaugeList->Count) * TAILLE_JAUGE_HAUT + \
        (SoundCtrlList->Count - 1) * (TAILLE_VOLUME + 2);

   SelectAudio = new  TSpeedButton(Princ);
   SelectAudio->Parent = Princ;
   SelectAudio->Height = 9;
   SelectAudio->Width = 9;
   SelectAudio->Left = 107;
   SelectAudio->Top = 5 + (GaugeList->Count) * TAILLE_JAUGE_HAUT + \
        (SoundCtrlList->Count - 1) * (TAILLE_VOLUME+2);

  // Adjust vertical window size to show the new control
  MainWindowHeight += VolCtrlBevel->Height-2;
    }
//---------------------------------------------------------------------------
// Delete all controls and gauges, free the memory
void __fastcall TPrinc::CleanUp()
  {
  while(GaugeList->Count > 0)
    {
    Gauge = (TJaugeMat*) GaugeList->First();
    delete Gauge;
    GaugeList->Delete(0);
    }

  while(SoundCtrlList->Count > 0)
    {
    Volume = (TTrackBar*) SoundCtrlList->First();
    delete Volume;
    SoundCtrlList->Delete(0);
    }

    if(((Princ->Affichage & DISPLAY_STATUS) != 0)) //    If (StatusBar != NULL)
      {
      delete StatusBar;
      }
}
//---------------------------------------------------------------------------
// Retrive system drives icons and put them in the component (ImageList1)
void __fastcall TPrinc::GetIcons()
  {
  char Fich[MAX_PATH];
  TIcon *Ico;

  Ico = new TIcon;
  GetSystemDirectory(Fich, MAX_PATH);
  lstrcat(Fich, "\\Shell32.dll");
  /*
  Ico.Handle:= LoadIcon(LoadLibrary(Fich), MAKEINTRESOURCE(4));  // 0: Closed folder
  ImageList1.AddIcon(Ico);
  Ico.Handle:= LoadIcon(LoadLibrary(Fich), MAKEINTRESOURCE(5));  // 1: Open folder
  ImageList1.AddIcon(Ico);
  */
  Ico->Handle= LoadIcon(LoadLibrary(Fich), MAKEINTRESOURCE(7));  // 2: Floppy
  ImageList1->AddIcon(Ico);
  Ico->Handle= LoadIcon(LoadLibrary(Fich), MAKEINTRESOURCE(9));  // 3: Hard drive
  ImageList1->AddIcon(Ico);
  Ico->Handle= LoadIcon(LoadLibrary(Fich), MAKEINTRESOURCE(10)); // 4: Network drive
  ImageList1->AddIcon(Ico);
  Ico->Handle= LoadIcon(LoadLibrary(Fich), MAKEINTRESOURCE(12)); // 5: CDROM DVD
  ImageList1->AddIcon(Ico);
  Ico->Handle= LoadIcon(LoadLibrary(Fich), MAKEINTRESOURCE(13)); // 6: RAM
  ImageList1->AddIcon(Ico);
  delete Ico;
  }

//---------------------------------------------------------------------------
// Refresh the RAM gauge
void __fastcall TPrinc::UpdateMemory(TJaugeMat *Jauge)
  {
  MEMORYSTATUS EtatMem;
  unsigned char i;
  bool Trouve;

  EtatMem.dwLength = sizeof(MEMORYSTATUS);
  GlobalMemoryStatus(&EtatMem);
  Gauge->Position = 100 * ((float) EtatMem.dwAvailPhys / EtatMem.dwTotalPhys);
  Gauge->Max = 100;
  Gauge->Libre = ConvertTaille(EtatMem.dwAvailPhys);
  Gauge->Capacite = ConvertTaille(EtatMem.dwTotalPhys);
  if (!Texture)
    {
    if (!Uniforme)
      {
      i = 0;
      Trouve = false;
      while (!Trouve && (i < (MAX_THRESHOLDS - NbSeuils)))
        {
        if (Jauge->Position < ValSeuil[i])
          {
          Trouve = true;
          Gauge->CoulBarre = Couleur[i];
          }
        else
          {
          i++;
          }
        }
      if (!Trouve)
        Gauge->CoulBarre = Couleur[i];
      }
    else
      Gauge->CoulBarre = Couleur[4];
    }
  }
//----------------------------------------------------------------------------
// Refresh the battery charge gauge
void __fastcall TPrinc::UpdateBattery(TJaugeMat *Jauge)
  {
  SYSTEM_POWER_STATUS EtatBatt;
  unsigned char i;
  bool Trouve;

  GetSystemPowerStatus(&EtatBatt);

  /* Battery charge status, can be a combination of the following values:
  Value	Meaning
  1	High
  2	Low
  4	Critical
  8	Charging
  128	No system battery
  255	Unknown status */

  if ((EtatBatt.BatteryFlag & 128) == 128)
    {
    Gauge->Label= txt_batt_non;
    }
  if ((EtatBatt.BatteryFlag & 1) == 1)
    {
    Gauge->Label= txt_batt_debit;
    }
  if ((EtatBatt.BatteryFlag & 2) == 2)
    {
    Gauge->Label= txt_batt_faible;
    }
  if ((EtatBatt.BatteryFlag & 4) == 4)
    {
    Gauge->Label= txt_batt_critique;
    }
  if (((EtatBatt.BatteryFlag & 128) != 128) && (((EtatBatt.BatteryFlag & 8) == 8) || ((EtatBatt.ACLineStatus & 1) == 1)))
    {
    Gauge->Label= txt_batt_charge;
    }
  if ((EtatBatt.BatteryFlag & 256) == 256)
    {
    Gauge->Label= txt_batt_erreur;
    }

  if (EtatBatt.BatteryLifePercent != 255)  // If we can perform a measurement...
    {
    Gauge->Pourcentage = true; // If a battery is installed display the "%" sign
    Gauge->Position = EtatBatt.BatteryLifePercent;
    Gauge->Max = 100;
    Gauge->Libre = ConvertHeure(EtatBatt.BatteryLifeTime);
    Gauge->Capacite = ConvertHeure(EtatBatt.BatteryFullLifeTime);
    }
  else
    {
    Gauge->Position = 0;  // Unable to make measurement, no battery ?
    Gauge->Pourcentage = false; // If ano battery installed hide the "%" sign
    }

   if (!Texture)
    {
    if (!Uniforme)
      {
      i = 0;
      Trouve = false;
      while (!Trouve && (i < (MAX_THRESHOLDS - NbSeuils)))
        {
        if (Jauge->Position < ValSeuil[i])
          {
          Trouve = true;
          Jauge->CoulBarre = Couleur[i];
          }
        else
          {
          i++;
          }
        }
      if (!Trouve)
        Jauge->CoulBarre = Couleur[i];
      }
    else
      Jauge->CoulBarre = Couleur[4];
    }
  }

//----------------------------------------------------------------------------
// Gets drive information from the jauge passed in parameters this function is
// called every "MAJNORM" seconds and one time at program startup
void __fastcall TPrinc::UpdateDrive(TJaugeMat *Gauge)
  {
  __int64 Bidon, TailTot, TailLib;
  unsigned char i;
  bool Trouve;
  char NomVol[13];
  NomVol[0] = '\0';
  AnsiString tmp;

  if (GetVolumeInformation(Gauge->Chemin, NomVol, 64, NULL, NULL, NULL, NULL, 0))
    {
    tmp = AnsiString("( ") + AnsiString(Gauge->Lecteur) + AnsiString(" ) ") + AnsiString(NomVol);
    if (tmp.Length() > 17)
      {
      // Too long labels are truncated
      tmp.SetLength(17);
      tmp[17] = '';
      }
    Gauge->Label = tmp;
    }
    else
      {
      // If we where unable to retreive label name
      Gauge->Label = AnsiString("( ") + AnsiString(Gauge->Lecteur) + AnsiString(" ) ");
      }
    if (Sysutils::GetDiskFreeSpaceEx(Gauge->Chemin, Bidon, TailTot, &TailLib))
      {
      Gauge->Max = 100; // In all cases
      if (Gauge->Tag != TAG_CD) // If drive is not a CDROM / DVD
        {
        if(InvertDisp)
          {
          // If we choose to show the space occuped
          Gauge->Position = 100 - (100 * ((float) TailLib / TailTot));
          }
          else
          {
          // If we choose to show the free space
          Gauge->Position = 100 * ((float) TailLib / TailTot);
          }


        Gauge->Libre = ConvertTaille(TailLib);
        Gauge->Capacite = ConvertTaille(TailTot);
        }
      else
        {
        Gauge->Capacite = ConvertTaille(TailLib);
        if (TailTot > MAX_CD_700)  // If a DVD and lesser than MAX_DVD value
          {
          Gauge->Libre = ConvertTaille(MAX_DVD - TailTot); //  (<= MAX_DVD)
          if (TailTot > MAX_DVD) // HUGE  stuff !! => the gauge is set to the
                                                  // capacity of the disk
            {
            Gauge->Capacite = ConvertTaille(TailTot);
            Gauge->Position = 0;
            Gauge->Libre = 0;
            }
          else
            {
            Gauge->Capacite = ConvertTaille(MAX_DVD);
            Gauge->Position = 100 * (1-((float) TailTot / MAX_DVD));
            }
          }
        else
          {
            if (TailTot > MAX_CD_655) // May be a 700 Mo CD: MAX_CD_700 Mo
            {
            Gauge->Libre = ConvertTaille(MAX_CD_700 - TailTot); // (<= MAX_CD_700 Mo)
            Gauge->Capacite = ConvertTaille(MAX_CD_700);
            Gauge->Position = 100 * (1-((float) TailTot / MAX_CD_700));
            }
          else                      // May be a 650 Mo CD +/- MAX_CD_655 Mo
            {
            Gauge->Libre = ConvertTaille(MAX_CD_655 - TailTot); // (<= MAX_CD_655Mo)
            Gauge->Capacite = ConvertTaille(MAX_CD_655);
            Gauge->Position = 100 * (1-((float) TailTot / MAX_CD_655));
            }
          }
        }

      if (!Texture)
        {
        if (!Uniforme)
          {
          i = 0;
          Trouve = false;
          while (!Trouve && (i < (MAX_THRESHOLDS - NbSeuils)))
            {
            if (Gauge->Position < ValSeuil[i])
              {
              Trouve = true;
              Gauge->CoulBarre = Couleur[i];
              }
            else
              {
              i++;
              }
            }
          if (!Trouve)
            Gauge->CoulBarre = Couleur[i];
          }
        else
          {
          Gauge->CoulBarre = Couleur[4];
          }

        }
      Gauge->Pourcentage = true; // Displays the "%" sign if non empty drive
      }
  // If GetDiskFreeSpaceEx fails, drive should be empty
    else
      {
      Gauge->Position = 0;
      Gauge->Max = 100; // In all cases
      Gauge->Libre = "";
      Gauge->Capacite = "";
      Gauge->Pourcentage = false;  // Hide the "%" sign if empty drive
      Gauge->Label = AnsiString("( ") + AnsiString(Gauge->Lecteur) + AnsiString(" ) ") + txt_lect_vide;
      }
  }

//---------------------------------------------------------------------------
// Displays the window near the edge it was beafore beeing hidden
void __fastcall TPrinc::UnHide(bool Remasquer)
  {
  int i;

  switch (ZoneMasque)
    {
    case HIDE_RIGHT:
      Left = Screen->Width - Width;
      Left = Left;
    break;
    case HIDE_LEFT:
      Left = 0;
    break;
    case HIDE_TOP:
      Top = 0;
    break;
    case HIDE_BOTTOM:
      Top = Screen->Height - Height;
    break;
    }
  Timer2->Enabled = Remasquer;
  OnMouseMove = NULL;
  if (GaugeList->Count > 0)
    {
    for(i = 0; i < GaugeList->Count; i++)
      ((TJaugeMat*) GaugeList->Items[i])->OnMouseMove = NULL;
    }
  }

//----------------------------------------------------------------------------
// Saves settings in the system registery
void __fastcall TPrinc::Save_cfg(void)
{
  TRegistry *Reg;

  // Init the registery
  Reg = new  TRegistry;

  // Open 'current user' key
  Reg->RootKey = HKEY_CURRENT_USER;

  // Open/create 'DSpace' key
  if (Reg->OpenKey(REG_CLE_APP, true))
  {
    // Write values in 'DSpace' key
    Reg->WriteBool(REG_VAL_ON_TOP, FormStyle == fsStayOnTop);
    Reg->WriteInteger(REG_VAL_LEFT, Left);
    Reg->WriteInteger(REG_VAL_TOP, Top);
    Reg->WriteInteger(REG_VAL_DISPLAY_CTRLS, Affichage);
    Reg->WriteBool(REG_VAL_TEXTURE, Texture);
    Reg->WriteBool(REG_UNIFORME, Uniforme);
    Reg->WriteInteger(REG_VAL_NSEUILS, NbSeuils);
    Reg->WriteInteger(REG_VAL_MAJNORM, Timer1->Interval);
    Reg->WriteBool(REG_VAL_MASQUER, bHideMainWindow);
    Reg->WriteInteger(REG_VAL_DURVIS, Timer2->Interval);
    Reg->WriteInteger(REG_VAL_ZONMASQ, ZoneMasque);
    Reg->WriteInteger(REG_VAL_COUL0, Couleur[0]);
    Reg->WriteInteger(REG_VAL_SEUIL0, ValSeuil[0]);
    Reg->WriteInteger(REG_VAL_COUL1, Couleur[1]);
    Reg->WriteInteger(REG_VAL_SEUIL1, ValSeuil[1]);
    Reg->WriteInteger(REG_VAL_COUL2, Couleur[2]);
    Reg->WriteInteger(REG_VAL_SEUIL2, ValSeuil[2]);
    Reg->WriteInteger(REG_VAL_COUL3, Couleur[3]);
    Reg->WriteInteger(REG_VAL_COUL4, Couleur[4]);
    Reg->WriteInteger(REG_VAL_TYPE_TEXTURE, Type_Texture);
    Reg->WriteBool(REG_VAL_EXPLORATEUR, Explorateur);
    Reg->WriteBool(REG_VAL_AUTODEMARRAGE, Autodemarrage);
    }
  delete Reg;

  // Create or remove key used to auto start the application
  TRegistry *Reg2 = new TRegistry();

  // Write in  HKEY_CURRENT_USER
  Reg2->RootKey = HKEY_CURRENT_USER;
  if (Reg2->OpenKey("Software\\Microsoft\\Windows\\CurrentVersion\\Run", false))
    {
    if (Autodemarrage == true)
      {
      AnsiString temp = GetCurrentDir();
      if (temp[temp.Length()] != '\\')
        {
        temp = temp + "\\";
        }
      Reg2->WriteString("DSpace.exe", temp + "DSpace.exe");
      }
    else
      {
      Reg2->DeleteValue("DSpace.exe");
      }
    }
  delete Reg2;
}

//---------------------------------------------------------------------------
// Reads the system registery to gets the settings of the application
// Also read the dspace.ini file (if exists) and therefore set the foreigns
// languages (than french). Sets dfaults values if the key was not created yet
// The ini file is never written by the application.
void __fastcall TPrinc::Load_cfg(void)
  {
  TRegistry *Reg;

  // Init the system registery
  Reg = new  TRegistry;

  // Open 'current user' key
  Reg->RootKey = HKEY_CURRENT_USER;

  // Open/create 'DSpace' key
  if (Reg->OpenKey(REG_CLE_APP, false))
    {
    if (Reg->ValueExists(REG_VAL_ON_TOP))
      if (Reg->ReadBool(REG_VAL_ON_TOP))
        FormStyle = fsStayOnTop;
      else
        FormStyle = fsNormal;
    else
      FormStyle = DEF_ON_TOP;
    if (Reg->ValueExists(REG_VAL_LEFT))
      Left = Reg->ReadInteger(REG_VAL_LEFT);
    else
      Left = (Screen->Width - Width) / 2;
    if (Reg->ValueExists(REG_VAL_TOP))
      Top = Reg->ReadInteger(REG_VAL_TOP);
    else
      Top = (Screen->Height - Height) / 2;
    if (Reg->ValueExists(REG_VAL_DISPLAY_CTRLS))
      Affichage = Reg->ReadInteger(REG_VAL_DISPLAY_CTRLS);
    else
      Affichage = DEF_DISPLAY_CTRLS;
    if (Reg->ValueExists(REG_VAL_TEXTURE))
      Texture = Reg->ReadBool(REG_VAL_TEXTURE);
    else
      Texture = DEF_SHADING_ON;
    if (Reg->ValueExists(REG_UNIFORME))
      Uniforme = Reg->ReadBool(REG_UNIFORME);
    else
      Uniforme = DEF_NO_THRESHOLDS;
    if (Reg->ValueExists(REG_VAL_NSEUILS))
      NbSeuils = Reg->ReadInteger(REG_VAL_NSEUILS);
    else
      NbSeuils = DEF_NB_THRESHOLDS;
    if (Reg->ValueExists(REG_VAL_COUL0))
      Couleur[0] = (TColor) Reg->ReadInteger(REG_VAL_COUL0);
    else
      Couleur[0] = DEF_COLOR0;
    if (Reg->ValueExists(REG_VAL_SEUIL0))
      ValSeuil[0] = Reg->ReadInteger(REG_VAL_SEUIL0);
    else
      ValSeuil[0] = DEF_THRESHOLD0;
    if (Reg->ValueExists(REG_VAL_COUL1))
      Couleur[1] = (TColor) Reg->ReadInteger(REG_VAL_COUL1);
    else
      Couleur[1] = DEF_COLOR1;
    if (Reg->ValueExists(REG_VAL_SEUIL1))
      ValSeuil[1] = Reg->ReadInteger(REG_VAL_SEUIL1);
    else
      ValSeuil[1] = DEF_THRESHOLD1;
    if (Reg->ValueExists(REG_VAL_COUL2))
      Couleur[2] = (TColor) Reg->ReadInteger(REG_VAL_COUL2);
    else
      Couleur[2] = DEF_COLOR2;
    if (Reg->ValueExists(REG_VAL_SEUIL2))
      ValSeuil[2] = Reg->ReadInteger(REG_VAL_SEUIL2);
    else
      ValSeuil[2] = DEF_THRESHOLD2;
    if (Reg->ValueExists(REG_VAL_COUL3))
      Couleur[3] = (TColor) Reg->ReadInteger(REG_VAL_COUL3);
    else
      Couleur[4] = DEF_COLOR4;
    if (Reg->ValueExists(REG_VAL_COUL4))
      Couleur[4] = (TColor) Reg->ReadInteger(REG_VAL_COUL4);
    else
      Couleur[4] = DEF_COLOR4;
    if (Reg->ValueExists(REG_VAL_TYPE_TEXTURE))
      Type_Texture = Reg->ReadInteger(REG_VAL_TYPE_TEXTURE);
    else
      Type_Texture = DEF_SHADING_TYPE;
    if (Reg->ValueExists(REG_VAL_MAJNORM))
      Timer1->Interval = Reg->ReadInteger(REG_VAL_MAJNORM);
    else
      Timer1->Interval = DEF_NORMAL_REFRESH_RATE;
    if (Reg->ValueExists(REG_VAL_MASQUER))
      bHideMainWindow = Reg->ReadBool(REG_VAL_MASQUER);
    else
      bHideMainWindow = DEF_AUTO_HIDE;
    if (Reg->ValueExists(REG_VAL_DURVIS))
      Timer2->Interval = Reg->ReadInteger(REG_VAL_DURVIS);
    else
      Timer2->Interval = DEF_VISIBILITY_TIME;
    if (Reg->ValueExists(REG_VAL_ZONMASQ))
      ZoneMasque = Reg->ReadInteger(REG_VAL_ZONMASQ);
    else
      ZoneMasque = DEF_HIDDING_ZONE;
    if (Reg->ValueExists(REG_VAL_EXPLORATEUR))
      Explorateur = Reg->ReadInteger(REG_VAL_EXPLORATEUR);
    else
      Explorateur = DEF_EXPLORER_BROWSER;
    if (Reg->ValueExists(REG_VAL_AUTODEMARRAGE))
      Autodemarrage = Reg->ReadInteger(REG_VAL_AUTODEMARRAGE);
    else
      Autodemarrage = DEF_AUTO_LAUNCH;
    }
  else // If the key is not created yet
    {
    FormStyle = DEF_ON_TOP;
    Left = (Screen->Width - Width) / 2;
    Top = (Screen->Height - Height) / 2;
    Affichage = DEF_DISPLAY_CTRLS;
    Texture = DEF_SHADING_ON;
    NbSeuils = DEF_NB_THRESHOLDS;
    Couleur[0] = DEF_COLOR0;
    ValSeuil[0] = DEF_THRESHOLD0;
    Couleur[1] = DEF_COLOR1;
    ValSeuil[1] = DEF_THRESHOLD1;
    Couleur[2] = DEF_COLOR2;
    ValSeuil[2] = DEF_THRESHOLD2;
    Couleur[3] = DEF_COLOR3;
    Couleur[4] = DEF_COLOR4;
    Timer1->Interval = DEF_NORMAL_REFRESH_RATE;
    bHideMainWindow = DEF_AUTO_HIDE;
    Timer2->Interval = DEF_VISIBILITY_TIME;
    ZoneMasque = DEF_HIDDING_ZONE;
    Explorateur = DEF_EXPLORER_BROWSER;
    Autodemarrage = DEF_AUTO_LAUNCH;
    }
  Timer2->Enabled = bHideMainWindow;
  delete Reg;

// Gets the language string codes

// Init the inifile component
TIniFile* ini;

ini = new TIniFile(ChangeFileExt( Application->ExeName, ".INI" ) );

// Windows error
txt_unknown_drive_type = ini->ReadString("Language", "\"Unable to determine the drive type\"", DEF_INI_VAL_TXT_BATTERY);

// Main window strings
txt_batteries     = ini->ReadString("Language", "\"Battery\"", DEF_INI_VAL_TXT_BATTERY);
txt_memoire       = ini->ReadString("Language", "\"Memory\"", DEF_INI_VAL_TXT_MEMORY);
txt_rafraich      = ini->ReadString("Language", "\"Db. click to refresh...\"", DEF_INI_VAL_TXT_NEED_REFRESH);
txt_lect_vide     = ini->ReadString("Language", "\"No disc in drive\"", DEF_INI_VAL_TXT_NO_DISK);
txt_batt_non      = ini->ReadString("Language", "\"No battery set\"", DEF_INI_VAL_TXT_NO_BATTERY);
txt_batt_debit    = ini->ReadString("Language", "\"Using battery...\"", DEF_INI_VAL_TXT_BATTERY_POWERED);
txt_batt_faible   = ini->ReadString("Language", "\"Low battery!\"", DEF_INI_VAL_TXT_LOW_BATTERY);
txt_batt_critique = ini->ReadString("Language", "\"Low batt critical!\"", DEF_INI_VAL_TXT_BATTERY_CRITICAL);
txt_batt_charge   = ini->ReadString("Language", "\"Battery charging...\"", DEF_INI_VAL_TXT_BATTERY_CHARGING);
txt_batt_erreur   = ini->ReadString("Language", "\"- Battery satus error! -\"", DEF_INI_VAL_TXT_BATTERY_STAT_ERROR);
txt_volume        = ini->ReadString("Language", "\"MAIN volume\"", DEF_INI_VAL_TXT_MAIN_VOL);
txt_vol_midi      = ini->ReadString("Language", "\"MIDI volume\"", DEF_INI_VAL_TXT_MIDI_VOL);
txt_vol_aux       = ini->ReadString("Language", "\"AUX. volume\"", DEF_INI_VAL_TXT_AUX_VOL);
txt_vol_micro     = ini->ReadString("Language", "\"MIC. volume\"", DEF_INI_VAL_TXT_MIC_VOL);
txt_vol_wav       = ini->ReadString("Language", "\"WAV volume\"", DEF_INI_VAL_TXT_WAV_VOL);
txt_vol_cd        = ini->ReadString("Language", "\"CD volume \"", DEF_INI_VAL_TXT_CD_VOL);
txt_mem_2_pourcent = ini->ReadString("Language", "\"Mem.\"", DEF_INI_VAL_TXT_MEM_2_PERCENT);
txt_batt_2_pourcent = ini->ReadString("Language", "\"BATT.\"", DEF_INI_VAL_TXT_BATT_2_PERCENT);
txt_suffixe_2_pourcent = ini->ReadString("Language", "\"< 2 %\"", DEF_INI_VAL_TXT_SUFFIX_2_PERCENT);

delete ini;
}

//----------------------------------------------------------------------------
void __fastcall TPrinc::Timer1s(TObject *Sender)
// Used to displays some infos in the status bar and to adjust volume controls
{
 unsigned long tmp, tmp1;
 int i;
 bool bDisplayInfosInStatusBar = 0;
 TTrackBar *potentiometer = new TTrackBar(Princ);

  // Get sound volume and then adjust volume controls
 if (SoundCtrlList->Count > 0)   // Limited for now to only WAV setting !!
    {
    for(i = 0; i < SoundCtrlList->Count; i++)
      {
      Volume = (TTrackBar*)SoundCtrlList->Items[i];
      waveOutGetVolume(NULL, &tmp);
      tmp1 = tmp  >> 16;    // Left channel
      Volume->Position = max(tmp1, tmp - (tmp1 << 16))/655;
      }
    }

//----------

 if(((Princ->Affichage & DISPLAY_STATUS) != 0))  // if (StatusBar != NULL)
  {
  StatusBar->SimpleText = "";
 if (GaugeList->Count > 0)
    {
    for(i = 0; i < GaugeList->Count; i++)
      {
      Gauge = (TJaugeMat*) GaugeList->Items[i];

      // Set a visual alarm if hard Drive(s), Memory or Battery fall below 2 %

      // Test if an empty drive or if a battery is installed
      // (if capacity != null, and low percentage)
      if ((Gauge->Position <= 2) && (Gauge->Capacite.Length() != 0))
        {
        if (Gauge->Tag != TAG_CD)  // not a CD / DVD
          {
          bDisplayInfosInStatusBar = true;

          // If current gauge is a battery
          if (Gauge->Lecteur == '@')
            {
            StatusBar->SimpleText = StatusBar->SimpleText + txt_mem_2_pourcent;
            }
          // If current jauge is the RAM
          else if (Gauge->Lecteur == '#')
            {
            StatusBar->SimpleText = StatusBar->SimpleText + txt_batt_2_pourcent;
            }
          else
            {
            StatusBar->SimpleText = StatusBar->SimpleText +
              " " + (AnsiString)Gauge->Lecteur + ": ";
            }
          }
        }
      }

      if (bDisplayInfosInStatusBar == true)  // If we have a message to display...
        {
        // Show the message
        StatusBar->SimpleText =  StatusBar->SimpleText + " " + txt_suffixe_2_pourcent;
        }
      }
  }
delete(potentiometer);
 }
//---------------------------------------------------------------------------

void __fastcall TPrinc::FormCreate(TObject *Sender)
{
  // Background color for icons drawing
  ImageList1->BkColor= clBtnFace;

  // Main window title
  Princ->Caption = TXT_VERSION;

  // Hide application's task bar button
  SetWindowLong(Application->Handle, GWL_EXSTYLE, WS_EX_TOOLWINDOW);

  // Avoid system CD request when using GetDiskFreeSpaceEx with a empty drive
  SetErrorMode(SEM_FAILCRITICALERRORS);

  // Create the lists
  GaugeList = new TList;
  SoundCtrlList = new TList;

  // Retrive parameters
  Load_cfg();
  GetIcons();

  // Displays gauges and controls
  DisplaysControls();
}
//---------------------------------------------------------------------------


